/*
	Understanding Water Activity Lab
*/

const DRAG_ARROW = `
				<style>
					 @keyframes drag-arrow-animation {
						0% {
							opacity: 0.0;
							left: -157px;
						}
						20% {
							opacity: 1.0;
						}
						80% {
							opacity: 1.0;
						}
						100% {
							opacity: 0.0;
							left: 43px;
						}
					 }
					.drag-arrow {
						animation-name: drag-arrow-animation;
						animation-duration: 1.0s;
						animation-iteration-count: infinite;
						animation-timing-function: linear;
						z-index: 10;
						position: absolute;
						top: -24px;
					}
				</style>
				<img src="images/Arrow.png" class="drag-arrow" />
`;

const DARK_BACKGROUND = `
	<div
		style="
		width: 800px;
		height: 600px;
		position: absolute;
		background-color: #000;
	">&nbsp;</div>
`;

//	Creates a particle system
//	Pass in a configuration object with:
//		root : a jQuery container to emit the particles from.
class Steam {
	constructor( config ) {
		this.root = config.root;
		this.emissionRate = 0;
	}
	start() {
		this.emissionRate = 2;
		this.animate();
	}
	stop() {
		this.emissionRate = 0;
	}
	animate() {
		if (this.emissionRate < 1) return;
		requestAnimationFrame( function() {
			this.animate();
		}.bind(this));
		for( var i = 0; i < this.emissionRate; i++ ) {
			let particle = $(`
					<img
						style="
							position: absolute;
							left: ${d(121)-60}px;
							top: 0px;
							opacity: 0.0;
							mix-blend-mode: screen;
							transition: opacity 0.25s, left 2s, top 2s, transform 2s;
							transform: scale( 2 );
						"
						src="images/particle.png"
						/>`);
			this.root.append( particle );
			setTimeout( function() {
				particle.remove();
			}, 2000 );
			setTimeout( function() {
				particle.css( 'opacity', 0 );
			}, 1750 );
			requestAnimationFrame( function() {
				particle
						.css( 'opacity', '0.05' )
						//.css('left', '0px') // (d(21)-11)+'px')
						.css('top',  -(d(140)+280)+'px')
						.css('transform', 'scale(3.0)')
			});
		}
	}
}

//	Required state: Pause menu
class PauseMenu extends PauseMenuState {

	constructor() {
		super( {
			chapters: [
				{
					label: '<b>M</b>ain Menu',
					key: 'm',
					action: function() {
						LAB.jumpTo( new TitleLabState() );
					},
				},
				{
					label: 'Chapter <b>1</b>: Introduction',
					key: '1',
					action: function() {
						LAB.jumpTo( new WelcomeState() );
					},
				},
				{
					label: 'Chapter <b>2</b>: Preparing Corn',
					key: '2',
					action: function() {
						LAB.jumpTo( new TodayWeWillTestWaterActivityOfCorn() );
					},
				},
				{
					label: 'Chapter <b>3</b>: Control Batch',
					key: '3',
					action: function() {
						LAB.jumpTo( new StartByMeasuringNewlyDriedCorn() );
					},
				},
				{
					label: 'Chapter <b>4</b>: Test Batch A',
					key: '4',
					action: function() {
						LAB.jumpTo( new FirstSampleA() );
					},
				},
				{
					label: 'Chapter <b>5</b>: Test Batch B',
					key: '5',
					action: function() {
						LAB.jumpTo( new StartSampleB() );
					},
				},
				{
					label: 'Chapter <b>6</b>: Test Batch C',
					key: '6',
					action: function() {
						LAB.jumpTo( new TakeThreeSamplesActivity() );
					},
				},
				{
					label: 'Chapter <b>7</b>: Conclusion',
					key: '7',
					action: function() {
						LAB.jumpTo( new AroundTheWorld() );
					},
				},
			],
		} );
	}

}

//	Required state: Title menu
class TitleLabState extends TitleScreen {

	constructor() {
		super( {
			placard: 'images/ControllingWaterActivityPlacard.png',
			title: 'Controlling Water Activity',
			firstState: new WelcomeState(),
			credits: 'credits.html',
			chapter: 1,
			audio: 'audio/0_introduction.mp3',
		} );
	}

}

//	CHAPTER ONE
class WelcomeState extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Welcome, lab assistant! Today, we'll be investigating
					the food safety and shelf life of sun-dried corn, or maize.
				</p>
				`,
			audio: 'audio/1_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new AncientPreservationMethod() );
			},
			chapter: 1,
		} );
	}
}

class AncientPreservationMethod extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Sun drying sweet corn is an ancient preservation method
					still used today. UV rays from the sun help to kill
					microbes on the corn, while drying lowers water activity
					within the kernels.
				</p>
				`,
			audio: 'audio/2_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new WhenWaterActivityLevelsAreLow() );
			},
		} );
	}
}

class WhenWaterActivityLevelsAreLow extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					When water activity levels are low, spoilage and
					disease-causing microbes can't grow and reproduce, since
					microbes, like other organisms, require water to survive.
					This slows spoilage and prolongs shelf life.
				</p>
				`,
			audio: 'audio/3_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new WaterActivityLevelThreshold() );
			},
		} );
	}
}

class WaterActivityLevelThreshold extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/4_Backdrop.png" />
				<div class="balloon without-portrait"
					style="
						right: 20px;
						top: 60px;
						width: 300px;
					"
				>
					<div class="dialogue">
						<p>
							When water activity is less than 0.6
							(a<sub>w</sub>&lt;0.6), almost all microbes,
							including bacteria, molds, and yeasts, stop
							growing, as seen in the Virtual Lab
							<em>Understanding Water Activity</em>.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new VegetablesAreUsuallyDriedFurther() );
			},
			onInfo: function() {
				LAB.push( new ReferenceOtherVirtualLab() );
			},
			audio: 'audio/4_narration.mp3',
			 });
	}
}

class ReferenceOtherVirtualLab extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					To learn more about how water activity affects microbial
					growth, see the Virtual Lab <em>Understanding Water
					Activity</em>.
				</p>
			`,
			audio: 'audio/4_InfoBox.mp3',
		} );
	}

}

class VegetablesAreUsuallyDriedFurther extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/4_Backdrop.png" />
				<img src="images/5_Backdrop.png" class="fade-in"/>
				<div class="balloon without-portrait"
					style="
						right: 20px;
						top: 60px;
						width: 300px;
					"
				>
					<div class="dialogue">
						<p>
							Vegetables are usually dried even further, to a
							water activity of 0.30 to 0.20, for safety
							and quality.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new TodayWeWillTestWaterActivityOfCorn() );
			},
			audio: 'audio/5_narration.mp3',
			 });
	}
}

//	CHAPTER TWO
class TodayWeWillTestWaterActivityOfCorn extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png" />
				<img src="images/6_Drying_Frame.png" style="
					left: 86px;
					bottom: 5px;" />
				<img src="images/6_Corn.png" style="
					left: 146px;
					bottom: 65px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Today we'll test the water activity of corn dried
							using traditional methods developed and used by
							Native Americans in the upper great plains of the
							United States.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new DeHuskCornActivity() );
			},
			audio: 'audio/6_narration.mp3',
			chapter: 2,
		});
	}
}

class DeHuskCornActivity extends DragActivity {
	constructor() {
		super( {
			html: `
				<img src="images/7_Backdrop.png" />
				<div id="corn" class="flipbook"
					style="
						left: 170px;
						top: 140px;
					">
				</div>
				<div id="thumb" class="thumb"></div>
				<div
					id="arrow-prompt"
					style="
						left: 430px;
						top: 350px;
						position: absolute;
						transform: rotate( 60deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Dried corn for this experiment was prepared by
							removing the silk and husk from ears of mature
							corn.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			drags: [
				new PointToPointDragAction( {
					thumbID: 'thumb',
					start: { x: 0, y: 0 },
					end: { x: 30, y: 60 },
					speed: 40,
					isDisjoint: true,
				} ),
				new PointToPointDragAction( {
					thumbID: 'thumb',
					start: { x: 30, y: 60 },
					end: { x: 60, y: 120 },
					speed: 40,
					isDisjoint: true,
				} ),
			],
			audio: 'audio/7_narration.mp3',
			 });
		this.root.find('#arrow-prompt-water').hide();
		this.corn = new FlipBook( this.root.find('#corn') );
		this.corn.loadFrames( 'images/7_CornCob', '.png', 7 );
		this.corn.showFrame( 1 );
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('#arrow-prompt').hide();
	}
	
	continueDrag( drag, x, y ) {
		super.continueDrag( drag, x, y );
		this.corn.showFrame( Math.round( x * 6 / 60.0 ) + 1 );
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		this.root.find('#arrow-prompt').show();
	}
	
	advance() {
		this.killInput();
		this.root.find('#arrow-prompt').remove();
		setTimeout( function() {
			LAB.switchTo( new BlanchActivity() );
		}, 1000 );
	}

}

class BlanchActivity extends DragActivity {
	constructor() {
		super( {
			html: `
				<img src="images/8_Backdrop.png" />
				<div id="thumb" class="thumb"></div>
				<img src="images/8_Pot.png" style="
					left: 130px;
					top: 260px;
				" />
				<img id="corn"
					src="images/8_Drop_Corn.png"
					style="
					left: 260px;
					top: 260px;
				" />
				<div id="steam" style="
					left: 260px;
					top: 260px;
					position: absolute;
				"></div>
				<img src="images/8_Pot_Front.png" style="
					left: 130px;
					top: 260px;
				" />
				<div
					id="arrow-prompt"
					style="
						left: 380px;
						top: 200px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 20px;
						top: 60px;
						width: 300px;
					"
				>
					<div class="dialogue">
						<p>
							Blanch the ears (boiling for ten minutes).
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: function() {
				LAB.push( new BlanchInfo() );
			},
			drags: [
				new PointToPointDragAction( {
					thumbID: 'corn',
					start: { x: 260, y: 50 },
					end: { x: 260, y: 260 },
					speed: 40,
					isDisjoint: true,
				} ),
			],
			audio: 'audio/8_narration.mp3',
			 });
		this.root.find('#arrow-prompt-water').hide();
		this.corn = new FlipBook( this.root.find('#corn') );
		this.corn.loadFrames( 'images/7_CornCob', '.png', 7 );
		this.corn.showFrame( 1 );
		
		this.particleSystem = new Steam( {
			root: this.root.find('#steam')
		});

	}
	
	arrive() {
		super.arrive();
		this.particleSystem.start();
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('#arrow-prompt').hide();
	}
	
	continueDrag( drag, x, y ) {
		super.continueDrag( drag, x, y );
		this.corn.showFrame( Math.round( x * 6 / 60.0 ) + 1 );
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		this.root.find('#arrow-prompt').show();
	}
	
	advance() {
		this.killInput();
		this.root.find('#arrow-prompt').remove();
		setTimeout( function() {
			LAB.switchTo( new RemoveKernelsActivity() );
		}, 1000 );
	}

}

class BlanchInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					Blanching is placing food (most often vegetables or fruits)
					into boiling water or steam for a short time. Blanching
					stops enzyme action that can cause loss of flavor, color,
					and texture. It also cleanses the food's surface of dirt
					and organisms, brightens the color, and helps slow the
					loss of vitamins.
				</p>
			`,
			audio: 'audio/8_InfoBox.mp3',
		} );
	}

}

class RemoveKernelsActivity extends DragActivity {
	constructor() {
		super( {
			html: `
				<img src="images/7_Backdrop.png" style="
					filter: blur(5px);"/>
				<img src="images/9_Corn_Back.png" style="
					left: 230px;
					top: 120px;
				" />
				<img id="spoon"
					src="images/9_Spoon.png"
					style="
					transform: rotate( -60deg );
					transform-origin: 50px 50px;
					left: 310px;
					top: 120px;
				" />
				<img id="kernels"
					src="images/9_Corn_Kernels.png" style="
					left: 245px;
					top: 145px;
					transform-origin: 61px 263px;
				" />
				<div
					id="arrow-prompt"
					style="
						left: 380px;
						top: 200px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 20px;
						top: 60px;
						width: 300px;
					"
				>
					<div class="dialogue">
						<p>
							Remove the kernels from the cob.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			drags: [
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 210, y: 30 },
					end: { x: 250, y: 450 },
					speed: 40,
					isDisjoint: true,
				} ),
			],
			audio: 'audio/9_narration.mp3',
			 });
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('#arrow-prompt').hide();
	}
	
	continueDrag( drag, x, y ) {
		super.continueDrag( drag, x, y );
		let ramp = (y - 30) / 420.0;
		let ang = -60 - ramp * 60;
		$('#spoon').css('transform', 'rotate(' + ang + 'deg)');
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		this.root.find('#arrow-prompt').show();
	}
	
	advance() {
		this.killInput();
		$('#kernels').css('transition', 'transform 1s ease' );
		$('#kernels').css('transform','rotate( -45deg ) translate( -35px, 35px )');
		this.root.find('#arrow-prompt').remove();
		setTimeout( function() {
			LAB.switchTo( new SunDryTheKernels() );
		}, 1500 );
	}

}

class SunDryTheKernels extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/10_Backdrop.png" />
				<img src="images/10_Corn_Rack.png" style="
					left: 86px;
					bottom: 65px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Sun dry the kernels outdoors between two screens
							for several days until they are brittle. Many
							Native American and Hispanic cultures have
							similar traditional methods of drying corn and
							other foods.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new FirstWeWillTestWaterActivity() );
			},
			audio: 'audio/10_narration.mp3',
			 });
	}
}

class FirstWeWillTestWaterActivity extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png" />
				<img src="images/10_Backdrop.png" id="outdoors" />
				<img src="images/11_Corn_Rack.png" style="
					left: 86px;
					bottom: 65px;" />
				<img id="old-rack"
					src="images/10_Corn_Rack.png" style="
					left: 86px;
					bottom: 65px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							First, we'll test water activity of the newly
							dried corn.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new WeWillPutInClothBags() );
			},
			audio: 'audio/11_narration.mp3',
			 });
	}
	
	arrive() {
		super.arrive();
		let oldRack = this.root.find('#old-rack, #outdoors');
		oldRack.css('transition','opacity 1s ease');
		requestAnimationFrame( function() {
			oldRack.css('opacity','0');
		});
	}
	
}

class WeWillPutInClothBags extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png" />
				<img src="images/12_Bags.png" style="
					left: 86px;
					bottom: 65px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							We'll put the corn in cloth bags and store each in
							a different environment. After 30 days, we'll test
							the corn again.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new TheThreeCases() );
			},
			onInfo: function() {
				LAB.push( new PillowcasesInfo() );
			},
			audio: 'audio/12_narration.mp3',
			 });
	}
	
}

class PillowcasesInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					Some people use pillowcases to store their dried corn.
					This allows for continued aeration. Glass jars, plastic
					containers, and plastic bags are also commonly used.
				</p>
			`,
			audio: 'audio/12_InfoBox.mp3',
		} );
	}

}

class TheThreeCases extends LabState {
	constructor() {
		super( {
			html: `
				${DARK_BACKGROUND}
				<div style="
					position: absolute;
					width: 700px;
					left: 50px;
					bottom: 100px;
					display: grid;
					grid-template-columns: 1fr 1fr 1fr;
					align-items: center;
					justify-items: center;"
				>
					<img
						id="sack-a"
						src="images/13_SackA.png" style="
						left: 86px;
						bottom: 65px;" />
					<img
						id="sack-b"
						src="images/13_SackB.png" style="
						left: 86px;
						bottom: 65px;" />
					<img
						id="sack-c"
						src="images/13_SackC.png" style="
						left: 86px;
						bottom: 65px;" />
				</div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							These environments are:
						</p>
						<ul>
							<li>Hanging in a dry pantry</li>
							<li>Sitting on a shelf against a wall in a dry
								pantry</li>
							<li>Sitting on a shelf in a damp basement</li>
						</ul>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new AccurateMeasurementsNeedCalibration() );
			},
			audio: 'audio/13_narration.mp3',
			 });
	}
	arrive() {
		super.arrive();
		this.root.find('#sack-a, #sack-b, #sack-c')
			.css('opacity', 0.25)
			.css('transition', 'opacity 1s ease');
		setTimeout( function() {
			this.root.find( '#sack-a' ).css('opacity', 1 );
		}.bind(this), 1000 );
		setTimeout( function() {
			this.root.find( '#sack-b' ).css('opacity', 1 );
		}.bind(this), 4000 );
		setTimeout( function() {
			this.root.find( '#sack-c' ).css('opacity', 1 );
		}.bind(this), 8000 );
	}
	
}

class AccurateMeasurementsNeedCalibration extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Accurate measurements always start with calibrated
					instruments. This procedure assumes that you have already
					calibrated the water activity meter.
				</p>
				`,
			audio: 'audio/14_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new WeMeasureBeforeStoring() );
			},
			onInfo: function() {
				LAB.push( new LearnToCalibrateInfo() );
			},
		} );
		this.root.find('img').css('filter','');
		this.root.append( `
			<img src="images/14_Meter.png" style="
					left: 500px;
					top: 350px;
				" />
		`);
	}
	
}

class LearnToCalibrateInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					Use the Virtual Lab <em>Understanding Water Activity</em>
					to learn how to calibrate a water activity meter. The
					accuracy of your measurements comes from calibrating –
					testing that the meter is measuring accurately – using a
					substance with known water activity.
				</p>
			`,
			audio: 'audio/14_InfoBox.mp3',
		} );
	}

}

class WeMeasureBeforeStoring extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png"/>
				<img src="images/15_Corn_Control.png" style="
					left: 100px;
					top: 350px;" />
				<img src="images/15_Meter.png" style="
					left: 350px;
					top: 300px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Before storing the corn, we test the water activity.
							After 30 days of storage, we test again to see if
							there was a change based on the storage
							environments.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new ThreeTimesForPrecision() );
			},
			audio: 'audio/15_narration.mp3',
			 });
	}
	
}

class ThreeTimesForPrecision extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png"/>
				<img src="images/15_Corn_Control.png" style="
					left: 100px;
					top: 350px;" />
				<img src="images/15_Meter.png" style="
					left: 350px;
					top: 300px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							For precision, we will need to take three samples
							each time we test a batch of corn.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new StartByMeasuringNewlyDriedCorn() );
			},
			onInfo: function() {
				LAB.push( new LabSafetyInfo() );
			},
			audio: 'audio/16_narration.mp3',
			 });
	}
	
}

class LabSafetyInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					In the lab, always follow safety rules.
				</p>
				<h2>Basic lab safety procedures</h2>
				<ul>
					<li>Wash your hands before and after handling materials.</li>
					<li>Wear gloves and safety glasses when handling chemicals.</li>
					<li>Don't eat or drink in the lab.</li>
					<li>Avoid wearing dangling jewelry or loose clothing, and pull
						back long hair.</li>
					<li>Follow equipment operating instructions.</li>
					<li>Clean equipment after use.</li>
				</ul>
			`,
			audio: 'audio/16_InfoBox.mp3',
		} );
	}

}

//	CHAPTER THREE
class StartByMeasuringNewlyDriedCorn extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/17_Backdrop.png"/>
				<img src="images/17_Corn_Control.png" style="
					left: 100px;
					top: 180px;" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Let's start by measuring a<sub>w</sub> for newly
							dried corn. This newly dried corn is our
							<em>control</em>.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new PutCornIntoGrinder() );
			},
			onInfo: function() {
				LAB.push( new ControlInfo() );
			},
			audio: 'audio/17_narration.mp3',
			chapter: 3,
		});
	}
	
}

class ControlInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					A <em>control</em> is a base treatment you can compare to
					experimental treatments. In this lab exercise, our control
					treatment is drying corn and measuring its water activity
					immediately. Our experimental treatments are storing
					dried corn for thirty days under varying conditions.
				</p>
			`,
			audio: 'audio/17_InfoBox.mp3',
		} );
	}

}

//	PutCornIntoGrinderActivity
//	Displays a scene where the user puts corn from a bag into a grinder.
//	Pass in a config object with:
//
//		cornImage - image for the corn bag
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class PutCornIntoGrinderActivity extends DragActivity {
	constructor( config = {} ) {
		super( { 
			html: `
				<img src="images/6_Backdrop.png" />
				<div id="grinder" class="flipbook"
					style="
						left: 500px;
						top: 275px;"></div>
				<div id="scoop" class="flipbook"
					style="
						left: 200px;
						top: 175px;
						transition: transform 0.5s ease;
						"></div>
				<img src="${config.cornImage}" style="
					left: 200px;
					top: 370px;
					"/>
					style="
						left: 200px;
						top: 275px;"></div>
				<div
					id="arrow-prompt-1"
					class="arrow-prompt"
					style="
						left: 280px;
						top: 200px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				<div
					id="arrow-prompt-2"
					class="arrow-prompt"
					style="
						left: 400px;
						top: 340px;
						position: absolute;
						transform: rotate( -30deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						${config.text}
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			drags: [
				new PointToPointDragAction( {
					thumbID: 'scoop',
					start: { x: 250, y: 200 },
					end: { x: 225, y: 350 },
					speed: 40,
					whenDone:function() {
						LAB.state.root.find('#scoop').css('transform', '');
						LAB.state.scoop.showFrame( 2 );
					},
					arrow: 'arrow-prompt-1',
				} ),
				new PointToPointDragAction( {
					thumbID: 'scoop',
					start: { x: 225, y: 350 },
					end: { x: 525, y: 230 },
					speed: 40,
					whenDone:function() {
						LAB.state.root.find('#scoop').css('transform', '');
						LAB.state.scoop.showFrame( 2 );
					},
					arch: -100,
					arrow: 'arrow-prompt-2',
				} ),
			],
			audio: config.audio,
		});
		this.grinder = new FlipBook( this.root.find('#grinder') );
		this.grinder.loadFrames( 'images/18_Grinder_', '.png', 2 );
		this.grinder.showFrame( 1 );
		this.scoop = new FlipBook( this.root.find('#scoop') );
		this.scoop.loadFrames( 'images/18_Scoop_', '.png', 2 );
		this.scoop.showFrame( 1 );
		this.root.find('.arrow-prompt').hide();
		this.root.find('#arrow-prompt-1').show();
		this.whenDone = config.onContinue;
	}
	
	arrive() {
		super.arrive();
		this.root.find('#scoop').css('transform', 'translate( -100px, 0px )');
		requestAnimationFrame( function() {
			this.root.find('#scoop').css('transform', 'rotate( -30deg )');
		}.bind(this));
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('.arrow-prompt').css('opacity',0);
		this.root.find('.arrow-prompt').hide();
		this.root.find('#' + drag.config.arrow).show();
	}
	
	continueDrag( drag, x, y ) {
		super.continueDrag( drag, x, y );
		if (drag.config.arch != null) {
			let ramp = (x - 225) / 300.0;
			let ease = Math.sin( ramp * Math.PI );
			let delta = Math.sin( ramp * Math.PI ) * drag.config.arch;
			$('#scoop > img').css('transform', 'translate( 0px, ' + delta + 'px )');
		}
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		if (complete) {
			if (drag.config.whenDone) drag.config.whenDone();
		}
		this.root.find('.arrow-prompt').css('opacity',1);
	}
	
	advance() {
		this.killInput();
		this.root.find('.arrow-prompt').remove();
		this.root.find('#scoop').css('transform', 'rotate(-30deg)');
		setTimeout( function() {
			this.grinder.showFrame( 2 );
			this.scoop.showFrame( 1 );
		}.bind(this), 250 );
		setTimeout( function() {
			this.root.find('#scoop').css('transform', 'rotate(0deg) translate(50px, -50px)');
		}.bind(this), 550 );
		setTimeout( function() {
			this.whenDone();
		}.bind(this), 1500 );
	}

}

class PutCornIntoGrinder extends PutCornIntoGrinderActivity {
	constructor() {
	
		super( {
			cornImage: 'images/15_Corn_Control.png',
			onContinue: function() {
				LAB.switchTo( new GrindTheCorn() );
			},
			onInfo: function() {
				LAB.push( new AvoidingAmbientMoistureInfo() );
			},
			text: `
				<p>
					Prepare the corn for testing. First, place a scoop
					of kernels into a small grinder.
				</p>
			`,
			audio: 'audio/18_narration.mp3',
		} );
	}
	
}

class AvoidingAmbientMoistureInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					To keep the sample from gaining or losing moisture before
					testing:
				</p>
				<ul>
					<li>Wear gloves, so your skin doesn't add moisture to the
					sample.</li>
					<li>Make sure testing is done in a room with a low ambient
					temperature.</li>
					<li>Use sample lids or parafilm to reduce air exposure in
					samples waiting to be tested.</li>
				</ul>
			`,
			audio: 'audio/18_InfoBox.mp3',
		} );
	}

}

//	GrindTheCornActivity
//	Displays a scene where the user grinds corn with a grinder.
//	Pass in a config object with:
//
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class GrindTheCornActivity extends LabState {
	constructor( config ) {
		super( {
			html: `
				<img src="images/19_Backdrop.png" />
				<img id="grinder"
					src="images/19_Grinder.png" style="
						left: 270px;
						bottom: 40px;"/>
				<div id="grind-button"
					style="
					width: 40px;
					height: 80px;
					left: 377px;
					top: 325px;
					cursor: pointer
					" onclick="LAB.state.advance();"></div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						${config.text}
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			audio: config.audio,
		});
		this.whenDone = config.onContinue;
	}
	
	advance() {
		this.killInput();
		let grinder = this.root.find( '#grinder' );
		grinder.css('transition', 'transform 0.05s ease');
		for( var i = 0; i < 50; i++ ) {
			setTimeout( function() {
				grinder.css('transform', 'translate( ' + (d(7)-4) + 'px, ' +
					(d(7)-4) + 'px) rotate(' + (d(7)-4) + 'deg)');
			}, i * 50 );
		}
		setTimeout( function() {
			this.whenDone();
		}.bind(this), 1500 );
	}

}

class GrindTheCorn extends GrindTheCornActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new TransferCornFromGrinder() );
			},
			text: `<p>Grind up the corn.</p>`,
			audio: 'audio/19_narration.mp3',
		} );
	}
}

//	TransferCornFromGrinderActivity
//	Displays a scene where the user transfers ground corn into a cup:
//
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class TransferCornFromGrinderActivity extends DragActivity {
	constructor( config ) {
		super( {
			html: `
				<img src="images/6_Backdrop.png" />
				<div id="dish" class="flipbook"
					style="
						left: 500px;
						top: 475px;"></div>
				<img src="images/20_Grinder_Powder.png" style="
					left: 200px;
					top: 300px;
					"/>
					style="
						left: 200px;
						top: 275px;"></div>
				<div id="scoop" class="flipbook"
					style="
						left: 200px;
						top: 175px;
						transition: transform 0.5s ease;
						"></div>
				<div
					id="arrow-prompt-1"
					class="arrow-prompt"
					style="
						left: 280px;
						top: 200px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				<div
					id="arrow-prompt-2"
					class="arrow-prompt"
					style="
						left: 400px;
						top: 360px;
						position: absolute;
						transform: rotate( 45deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			audio: config.audio,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.info,
			drags: [
				new PointToPointDragAction( {
					thumbID: 'scoop',
					start: { x: 250, y: 200 },
					end: { x: 225, y: 270 },
					speed: 40,
					whenDone:function() {
						LAB.state.root.find('#scoop').css('transform', '');
						LAB.state.scoop.showFrame( 2 );
					},
					arrow: 'arrow-prompt-1',
				} ),
				new PointToPointDragAction( {
					thumbID: 'scoop',
					start: { x: 225, y: 270 },
					end: { x: 525, y: 430 },
					speed: 40,
					whenDone:function() {
						LAB.state.root.find('#scoop').css('transform', '');
						LAB.state.scoop.showFrame( 2 );
					},
					arch: -100,
					arrow: 'arrow-prompt-2',
				} ),
			],
			 });
		this.dish = new FlipBook( this.root.find('#dish') );
		this.dish.loadFrames( 'images/20_Dish_', '.png', 2 );
		this.dish.showFrame( 1 );
		this.scoop = new FlipBook( this.root.find('#scoop') );
		this.scoop.loadFrames( 'images/20_Scoop_', '.png', 2 );
		this.scoop.showFrame( 1 );
		this.root.find('.arrow-prompt').hide();
		this.root.find('#arrow-prompt-1').show();
		this.whenDone = config.onContinue;
	}
	
	arrive() {
		super.arrive();
		this.root.find('#scoop').css('transform', 'translate( -100px, 0px )');
		requestAnimationFrame( function() {
			this.root.find('#scoop').css('transform', 'rotate( -30deg )');
		}.bind(this));
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('.arrow-prompt').css('opacity',0);
		this.root.find('.arrow-prompt').hide();
		this.root.find('#' + drag.config.arrow).show();
	}
	
	continueDrag( drag, x, y ) {
		super.continueDrag( drag, x, y );
		if (drag.config.arch != null) {
			let ramp = (x - 225) / 300.0;
			let ease = Math.sin( ramp * Math.PI );
			let delta = Math.sin( ramp * Math.PI ) * drag.config.arch;
			$('#scoop > img').css('transform', 'translate( 0px, ' + delta + 'px )');
		}
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		if (complete) {
			if (drag.config.whenDone) drag.config.whenDone();
		}
		this.root.find('.arrow-prompt').css('opacity',1);
	}
	
	advance() {
		this.killInput();
		this.root.find('.arrow-prompt').remove();
		this.root.find('#scoop').css('transform', 'rotate(-30deg)');
		setTimeout( function() {
			this.dish.showFrame( 2 );
			this.scoop.showFrame( 1 );
		}.bind(this), 250 );
		setTimeout( function() {
			this.root.find('#scoop').css('transform', 'rotate(0deg) translate(50px, -50px)');
		}.bind(this), 550 );
		setTimeout( function() {
			this.whenDone();
		}.bind(this), 1500 );
	}

}

class TransferCornFromGrinder extends TransferCornFromGrinderActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new SettleAndPat() );
			},
			text: `
				<p>
					Now, add a bit of this mixture to the sample dish,
					enough to cover the bottom of the dish completely,
					but don't fill the dish more than half full.
				</p>
			`,
			audio: 'audio/20_narration.mp3',
		} );
	}
}

//	SettleAndPatActivity
//	Displays a scene where the user settles ground corn.
//
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class SettleAndPatActivity extends DragActivity {
	constructor( config ) {
		super( {
			html: `
				${DARK_BACKGROUND}
				<div id="dish" class="flipbook"
					style="
						left: 225px;
						top: 300px;"></div>
				<div
					id="spoon"
					style="
						position: absolute;
						left: 400px;
						top: 270px;
						transform: rotate( 0deg );">
					<img
						src="images/21_Spoon.png"
						style="
							position: absolute;
							left: -100px;
							top: -70px;
							transform-origin: 100px 70px;
							transform: translate( 0px, -500px ) rotate( 15deg );
							transition: transform 0.5s ease;
						"
						/>
				</div>
				<div
					id="arrow-prompt-1"
					class="arrow-prompt"
					style="
						left: 300px;
						top: 300px;
						position: absolute;
						transform: rotate( 180deg );
					">
					${DRAG_ARROW}
				</div>
				<div
					id="arrow-prompt-2"
					class="arrow-prompt"
					style="
						left: 500px;
						top: 300px;
						position: absolute;
						transform: rotate( 0deg );
					">
					${DRAG_ARROW}
				</div>
				<div
					id="arrow-prompt-3"
					class="arrow-prompt"
					style="
						left: 400px;
						top: 300px;
						position: absolute;
						transform: rotate( -90deg );
					">
					${DRAG_ARROW}
				</div>
				<div
					id="arrow-prompt-4"
					class="arrow-prompt"
					style="
						left: 400px;
						top: 300px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			audio: config.audio,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			drags: [
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 225, y: 300 },
					end: { x: 125, y: 300 },
					speed: 40,
					arrow: 'arrow-prompt-1',
				} ),
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 125, y: 300 },
					end: { x: 325, y: 300 },
					speed: 40,
					dishFrame: 2,
					arrow: 'arrow-prompt-2',
				} ),
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 325, y: 300 },
					end: { x: 125, y: 300 },
					speed: 40,
					arch: -100,
					arrow: 'arrow-prompt-1',
				} ),
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 125, y: 300 },
					end: { x: 325, y: 300 },
					speed: 40,
					dishFrame: 3,
					arrow: 'arrow-prompt-2',
				} ),
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 325, y: 300 },
					end: { x: 125, y: 300 },
					speed: 40,
					arch: -100,
					arrow: 'arrow-prompt-1',
				} ),
				new PointToPointDragAction( {
					thumbID: 'dish',
					start: { x: 125, y: 300 },
					end: { x: 225, y: 300 },
					speed: 40,
					arrow: 'arrow-prompt-2',
					isDisjoint: true,
					showSpoon: true,
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 270 },
					end: { x: 400, y: 370 },
					speed: 40,
					arrow: 'arrow-prompt-4',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 370 },
					end: { x: 400, y: 270 },
					speed: 40,
					arrow: 'arrow-prompt-3',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 270 },
					end: { x: 400, y: 370 },
					speed: 40,
					arrow: 'arrow-prompt-4',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 370 },
					end: { x: 400, y: 270 },
					speed: 40,
					arrow: 'arrow-prompt-3',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 270 },
					end: { x: 400, y: 370 },
					speed: 40,
					arrow: 'arrow-prompt-4',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 370 },
					end: { x: 400, y: 270 },
					speed: 40,
					arrow: 'arrow-prompt-3',
				} ),
				new PointToPointDragAction( {
					thumbID: 'spoon',
					start: { x: 400, y: 270 },
					end: { x: 400, y: 370 },
					speed: 40,
					whenDone:function() {
						LAB.state.root.find('#spoon img')
							.css('transform', 'translate( 0px, -500px ) rotate( 15deg )');
					},
					arrow: 'arrow-prompt-4',
				} ),
			],
			 });
		this.dish = new FlipBook( this.root.find('#dish') );
		this.dish.loadFrames( 'images/21_Cup_', '.png', 3 );
		this.dish.showFrame( 1 );
		this.root.find('.arrow-prompt').hide();
		this.root.find('#arrow-prompt-1').show();
		this.whenDone = config.onContinue;
	}
	
	dragOnDeck( drag, isAnimating ) {
		if (isAnimating == true) {
			this.root.find('.arrow-prompt').remove();
			return;
		}
		this.root.find('.arrow-prompt').hide();
		if (drag.config.arrow) this.root.find('#' + drag.config.arrow).show();
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		if (drag.config.dishFrame != null) {
			this.dish.showFrame( drag.config.dishFrame );
		}
		if (complete) {
			if (drag.config.whenDone) drag.config.whenDone();
			if (drag.config.showSpoon) {
				this.root.find('#spoon img').css('transform', '');
			}
		}
	}
	
	advance() {
		this.killInput();
		this.root.find('.arrow-prompt').remove();
		setTimeout( function() {
			this.whenDone();
		}.bind( this ), 1500 );
	}

}

class SettleAndPat extends SettleAndPatActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new PutDishInMeter() );
			},
			text: `
				<p>
					Shake the dish gently to get an even distribution,
					then press down the corn with the backside of a
					clean, dry spoon.
				</p>
			`,
			audio: 'audio/21_narration.mp3',
		} );
	}
}

//	PutDishInMeterActivity
//	Displays a scene where the user transfers ground corn into a meter:
//
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class PutDishInMeterActivity extends DragActivity {
	constructor( config ) {
		super( {
			html: `
				<img src="images/19_Backdrop.png" />
				<div id="meter" class="flipbook" style="
					left: 50px;
					top: 250px;
				"></div>
				<img id="cup" src="images/22_Cup.png" style="
					left: 600px;
					top: 250px;
					transform: scale( 0.8 );
				" />
				<div
					id="arrow-prompt"
					style="
						left: 450px;
						top: 200px;
						position: absolute;
						transform: rotate( 90deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			drags: [
				new PointToPointDragAction( {
					thumbID: 'cup',
					start: { x: 600, y: 250 },
					end: { x: 400, y: 340 },
					speed: 40,
					isDisjoint: true,
				} ),
			],
			audio: config.audio,
		});
		this.meter = new FlipBook( this.root.find('#meter') );
		this.meter.loadFrames( 'images/22_Meter_', '.png', 3 );
		this.meter.showFrame( 1 );
		this.whenDone = config.onContinue;
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('#arrow-prompt').hide();
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		this.root.find('#arrow-prompt').show();
	}
	
	advance() {
		this.killInput();
		this.root.find('#arrow-prompt').remove();
		this.root.find('#cup').remove();
		this.meter.showFrame( 2 );
		setTimeout( function() {
			this.meter.showFrame( 3 );
		}.bind(this), 500 );
		setTimeout( function() {
			this.whenDone();
		}.bind(this), 1500 );
	}

}

class PutDishInMeter extends PutDishInMeterActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new WhenItBeepsItIsDone() );
			},
			text: `
				<p>
					Open the water activity meter drawer and
					carefully place the sample dish inside.
				</p>
			`,
			audio: 'audio/22_narration.mp3',
		} );
	}
}

//	WhenItBeepsItIsDoneActivity
//	Lets the user wait for the water activity meter to beep.
//
//		onContinue - 
//		onInfo - 
//		text - Speech balloon contents
//		audio - Audio to play.
//		
class WhenItBeepsItIsDoneActivity extends LabState {
	constructor( config ) {
		super( {
			html: `
				<img src="images/1_Backdrop.png" />
				<img src="images/23_Meter.png" style="
						left: 270px;
						bottom: 100px;"/>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: config.onContinue,
			onInfo: config.onInfo,
			audio: config.audio,
		});
	}
	
}

class WhenItBeepsItIsDone extends WhenItBeepsItIsDoneActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebook() );
			},
			text: `
				<p>
					When the meter beeps, the measurement is complete.
				</p>
			`,
			audio: 'audio/23_narration.mp3',
		} );
	}
}

//	Shows a pad of paper with notes on it.
//		text, audio, onContinue
//		image - The image to display for the notes.
//		left, top - Where, relative to the pad, to put the image,
//				defaults to a nice upper-left placement.
//		depth - Defaults to zero.  If positive, doesn't lift the paper as high.
class ShowNotebook extends LabState {
	constructor( config ) {
		if (config.left == null) config.left = 175;
		if (config.top == null) config.top = 175;
		if (config.depth == null) config.depth = 0;
		super( {
			html: `
				<img src="images/1_Backdrop.png" />
				<img src="images/23_Meter.png" style="
						left: 270px;
						bottom: 100px;"/>
				<div id="pad" style="
					position: absolute;
					left: 0px;
					top: ${config.depth}px;
					width: 800px;
					transition: transform 1s ease;
					transform: translateY( 400px );
				">
					<img src="images/24_PadOfPaper.png" style="
						position: absolute;
						left: 0px;
						top: 0px;
					"/>
					<img src="${config.image}" style="
						position: absolute;
						left: ${config.left}px;
						top: ${config.top}px;
					"/>
					<img id="hand"
						src="images/24_Writing_Hand.png" style="
						position: absolute;
						transition: transform 1s ease;
						right: -400px;
						top: 200px;
						transform: translate( -300px, 0px );
					"/>
				</div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			audio: config.audio,
		});
		this.whenDone = config.onContinue;
	}
	
	advance() {
		this.whenDone();
	}
	
	arrive() {
		super.arrive();
		requestAnimationFrame( function() {
			this.root.find('#pad').css('transform','');
			this.root.find('#hand').css('transform','');
		}.bind(this));
	}
	
}

class RecordItInYourNotebook extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/24_Writing.png',
			onContinue: function() {
				LAB.switchTo( new DiscardSample() );
			},
			text: `
				<p>
					Record the water activity measurement and the
					temperature in your lab notebook.
				</p>
			`,
			audio: 'audio/24_narration.mp3',
			depth: 100,
		});
	}
}

//	DiscardSampleActivity
//	Prompts the user to discard a used sample of corn.
class DiscardSampleActivity extends DragActivity {
	constructor( config ) {
		super( {
			html: `
				<img src="images/19_Backdrop.png" />
				<div id="meter" class="flipbook" style="
					left: 50px;
					top: 250px;
				"></div>
				<img src="images/40_Trash.png" style="
					right: 0px;
					bottom: -50px;
				" />
				<img id="cup" src="images/22_Cup.png" style="
					left: 600px;
					top: 250px;
					transform: scale( 0.8 );
				" />
				<div
					id="arrow-prompt"
					style="
						left: 450px;
						top: 400px;
						position: absolute;
						transform: rotate( 45deg );
					">
					${DRAG_ARROW}
				</div>
				${DRAG_AREA}
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			onInfo: config.onInfo,
			drags: [
				new PointToPointDragAction( {
					thumbID: 'cup',
					start: { x: 400, y: 340 },
					end: { x: 550, y: 440 },
					speed: 40,
					isDisjoint: true,
				} ),
			],
			audio: config.audio,
		});
		this.meter = new FlipBook( this.root.find('#meter') );
		this.meter.loadFrames( 'images/22_Meter_', '.png', 3 );
		this.meter.showFrame( 1 );
		this.whenDone = config.onContinue;
	}
	
	startDrag( drag, x, y ) {
		super.startDrag( drag, x, y );
		this.root.find('#arrow-prompt').hide();
	}
	
	endDrag( drag, x, y, complete ) {
		super.endDrag( drag, x, y, complete );
		this.root.find('#arrow-prompt').show();
	}
	
	advance() {
		this.killInput();
		this.root.find('#arrow-prompt').remove();
		this.root.find('#cup')
			.css('transition', 'transform 1s linear')
			.css('transform','scale( 0.8 ) translate(100px,375px) rotate(60deg)');
		setTimeout( function() {
			this.meter.showFrame( 3 );
		}.bind(this), 500 );
		setTimeout( function() {
			this.whenDone();
		}.bind(this), 1200 );
	}

}

class DiscardSample extends DiscardSampleActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new TakeTwoMoreSamples() );
			},
			text: `
				<p>
					Then, dispose of the used sample and close the drawer.
				</p>
			`,
			audio: 'audio/40_narration.mp3',
		} );
	}
}

//	TakeTwoSamplesActivity
//	Shows two samples being performed.
class TakeTwoSamplesActivity extends LabState {
	constructor( config ) {
		super( {
			html: `
				<style>
					.sample-scene {
						position: absolute;
						left: 0px;
						transition: left 10s ease;
					}
					.sample-scene > * {
						position: absolute;
						transition: all 0.5s ease;
					}
					#scene-two {
						top: 300px;
					}
				</style>
				<div id="scene-one" class="sample-scene"></div>
				<div id="scene-two" class="sample-scene"></div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">${config.text}</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.state.advance();
			},
			audio: config.audio,
		});
		this.animations = [
			new GrindAnimation( this.root.find('#scene-one') ),
			new GrindAnimation( this.root.find('#scene-two') ),
		];
		this.whenDone = config.onContinue;
	}
	
	advance() {
		this.whenDone();
	}
	
	arrive() {
		super.arrive();
		this.animations[0].animate();
		setTimeout( function() { this.animations[1].animate(); }.bind(this), 6000 );
	}
	
}

class TakeTwoMoreSamples extends TakeTwoSamplesActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookNumberTwo() );
			},
			text: `
				<p>
					Test two more samples from the newly dried corn
					(the control).
				</p>
			`,
			audio: 'audio/25_narration.mp3',
		} );
		this.root.find('.corn-bag').attr('src','images/25_Corn.png')
	}
}

class GrindAnimation {
	constructor( root ) {
		this.root = root;
		this.grinder = new FlipBook( this.root.find( '.grinder' ));
		this.root.append( `
			<img src="images/25_Backdrop.png" />
			<img src="images/25_Corn_No_Label.png"
				class="corn-bag"
				style="left: 50px; top: 175px;" />
			<div class="grinder flipbook"
				style="left: 225px; top: 140px;"></div>
			<div class="meter flipbook"
				style="left: 500px; top: 125px;"></div>
			<div class="cup flipbook"
				style="left: 375px; top: 240px;"></div>
			<div class="scoop flipbook"
				style="left: -100px; top: 125px;"></div>
		`);
		this.grinder = new FlipBook( this.root.find('.grinder'));
		this.grinder.loadFrames('images/25_Grinder_', '.png', 4 );
		this.grinder.showFrame( 1 );
		this.cup = new FlipBook( this.root.find('.cup' ));
		this.cup.loadFrames('images/25_Cup_', '.png', 2 );
		this.cup.showFrame( 1 );
		this.meter = new FlipBook( this.root.find( '.meter' ));
		this.meter.loadFrames('images/25_Meter_', '.png', 3 );
		this.meter.showFrame( 1 );
		this.scoop = new FlipBook( this.root.find( '.scoop' ));
		this.scoop.loadFrames('images/25_Scoop_', '.png', 3 );
		this.scoop.showFrame( 1 );
		
		let grinder = this.grinder;
		let cup = this.cup;
		let meter = this.meter;
		let scoop = this.scoop;
		
		this.actions = [
			{
				time: 0,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '70px')
						.css('transform', 'rotate( -20deg )' );
				}.bind(this)
			},
			{
				time: 500,
				action: function() {
					this.root.find( '.scoop' )
						.css('top', '150px')
						.css('transform', 'rotate( 0deg )' );
				}.bind(this)
			},
			{
				time: 750,
				action: function() {
					this.scoop.showFrame( 2 );
				}.bind(this)
			},
			{
				time: 1000,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '250px')
						.css('top', '100px');
				}.bind(this)
			},
			{
				time: 1250,
				action: function() {
					this.root.find( '.scoop' )
						.css('transform','rotate(-20deg)');
				}.bind(this)
			},
			{
				time: 1250,
				action: function() {
					grinder.showFrame( 2 );
					scoop.showFrame( 1 );
				}.bind(this)
			},
			{
				time: 1500,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '120px')
						.css('top', '240px')
						.css('transform', 'rotate( 10deg )' );
				}.bind(this)
			},
			{
				time: 2000,
				action: function() {
					grinder.showFrame( 3 );
					this.root.find( '.grinder' )
						.css('transition', '');
				}.bind(this)
			},
			{
				time: 2100,
				action: function() {
					this.root.find( '.grinder' ).css('left', '220px');
				}.bind(this)
			},
			{
				time: 2200,
				action: function() {
					this.root.find( '.grinder' ).css('left', '230px');
				}.bind(this)
			},
			{
				time: 2300,
				action: function() {
					this.root.find( '.grinder' ).css('left', '220px');
				}.bind(this)
			},
			{
				time: 2400,
				action: function() {
					this.root.find( '.grinder' ).css('left', '230px');
				}.bind(this)
			},
			{
				time: 2500,
				action: function() {
					this.root.find( '.grinder' ).css('left', '220px');
				}.bind(this)
			},
			{
				time: 2600,
				action: function() {
					this.root.find( '.grinder' ).css('left', '225px');
					this.root.find( '.scoop' )
						.css('left', '250px')
						.css('top', '100px')
						.css('transform','rotate(-20deg)');
					grinder.showFrame( 4 );
				}.bind(this)
			},
			{
				time: 3100,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '230px')
						.css('top', '110px')
						.css('transform','rotate(0deg)');
					scoop.showFrame( 3 );
				}.bind(this)
			},
			{
				time: 3600,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '360px')
						.css('top', '190px')
						.css('transform','rotate(0deg)');
					scoop.showFrame( 3 );
				}.bind(this)
			},
			{
				time: 4100,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '370px')
						.css('top', '220px')
						.css('transform','rotate(-20deg)');
					scoop.showFrame( 1 );
					cup.showFrame( 2 );
				}.bind(this)
			},
			{
				time: 4600,
				action: function() {
					this.root.find( '.scoop' )
						.css('left', '330px')
						.css('top', '230px')
						.css('transform','rotate(20deg)');
					this.root.find( '.cup' )
						.css('left', '695px')
						.css('top', '180px')
						.css('transform', 'scale( 0.8 )');
				}.bind(this)
			},
			{
				time: 5100,
				action: function() {
					this.root.find( '.cup' ).remove();
					this.meter.showFrame( 2 );
				}.bind(this)
			},
			{
				time: 5600,
				action: function() {
					this.meter.showFrame( 3 );
				}.bind(this)
			},
		];
		
		this.timeStamp = 0;
		
	}
	
	animate() {
		requestAnimationFrame( function( time ) { this.advance( time ); }.bind( this ) );
	}
	
	advance( time ) {
		if (this.actions.length == 0) return;
		requestAnimationFrame( function( time ) { this.advance(time); }.bind(this) );
		if (this.startTime == null) this.startTime = time;
		let elapsed = time - this.startTime;
		let action = this.actions[0];
		if (action.time > elapsed) return;
		this.actions.shift();
		action.action();
	}
}

class RecordItInYourNotebookNumberTwo extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/26_Notes.png',
			left: 125,
			top: 75,
			onContinue: function() {
				LAB.switchTo( new AverageTheSamples() );
			},
			onInfo: function() {
				LAB.push( new SurfaceTemperatureInfo() );
			},
			text: `
				<p>
					Record the water activity measurement and the
					temperature in your lab notebook.
				</p>
			`,
			audio: 'audio/26_narration.mp3',
		});
	}
}

class SurfaceTemperatureInfo extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					The temperature reading is additional information. It
					refers to the sample's surface temperature at the
					time the water activity was taken.
				</p>
			`,
			audio: 'audio/26_InfoBox.mp3',
		} );
	}

}

class AverageTheSamples extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/27_Notes.png',
			left: 175,
			top: 125,
			onContinue: function() {
				LAB.switchTo( new AveragingIncreasesPrecision() );
			},
			text: `
				<p>
					Average the three control samples.
				</p>
			`,
			audio: 'audio/27_narration.mp3',
		});
	}
}

class AveragingIncreasesPrecision extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					By taking these multiple measurements and averaging them,
					you increase the precision, or reproducibility, of your
					measurements.
				</p>
				`,
			audio: 'audio/28_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new CornWaterActivity() );
			},
		} );
	}
}

class CornWaterActivity extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png" />
				<img src="images/29_Corn.png" style="
					left: 100px;
					top: 350px;
				"/>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							The dried corn starts out around water activity
							0.25 (a<sub>w</sub> = 0.25). That's within
							recommended levels of water activity (0.2 to 0.3)
							for dried vegetables.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new WaitThirtyDays() );
			},
			audio: 'audio/29_narration.mp3',
		});
	}
	
}

class WaitThirtyDays extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/6_Backdrop.png" style="filter: blur( 5px );"/>
				<img src="images/29_Corn.png" style="
					left: 100px;
					top: 350px;
					filter: blur( 5px );
				"/>
				${DARKENING_OVERLAY}
				<img src="images/30_Calendar.png" style="
					left: 185px;
					top: 150px;
				"/>

				<img id="day-1" src="images/30_X.png" style="left: 383px; top: 245px;" />
				<img id="day-2" src="images/30_X.png" style="left: 440px; top: 245px;" />
				<img id="day-3" src="images/30_X.png" style="left: 497px; top: 245px;" />
				<img id="day-4" src="images/30_X.png" style="left: 554px; top: 245px;" />

				<img id="day-5" src="images/30_X.png" style="left: 212px; top: 289px;" />
				<img id="day-6" src="images/30_X.png" style="left: 269px; top: 289px;" />
				<img id="day-7" src="images/30_X.png" style="left: 326px; top: 289px;" />
				<img id="day-8" src="images/30_X.png" style="left: 383px; top: 289px;" />
				<img id="day-9" src="images/30_X.png" style="left: 440px; top: 289px;" />
				<img id="day-10" src="images/30_X.png" style="left: 497px; top: 289px;" />
				<img id="day-11" src="images/30_X.png" style="left: 554px; top: 289px;" />

				<img id="day-12" src="images/30_X.png" style="left: 212px; top: 332px;" />
				<img id="day-13" src="images/30_X.png" style="left: 269px; top: 332px;" />
				<img id="day-14" src="images/30_X.png" style="left: 326px; top: 332px;" />
				<img id="day-15" src="images/30_X.png" style="left: 383px; top: 332px;" />
				<img id="day-16" src="images/30_X.png" style="left: 440px; top: 332px;" />
				<img id="day-17" src="images/30_X.png" style="left: 497px; top: 332px;" />
				<img id="day-18" src="images/30_X.png" style="left: 554px; top: 332px;" />

				<img id="day-19" src="images/30_X.png" style="left: 212px; top: 376px;" />
				<img id="day-20" src="images/30_X.png" style="left: 269px; top: 376px;" />
				<img id="day-21" src="images/30_X.png" style="left: 326px; top: 376px;" />
				<img id="day-22" src="images/30_X.png" style="left: 383px; top: 376px;" />
				<img id="day-23" src="images/30_X.png" style="left: 440px; top: 376px;" />
				<img id="day-24" src="images/30_X.png" style="left: 497px; top: 376px;" />
				<img id="day-25" src="images/30_X.png" style="left: 554px; top: 376px;" />

				<img id="day-26" src="images/30_X.png" style="left: 212px; top: 419px;" />
				<img id="day-27" src="images/30_X.png" style="left: 269px; top: 419px;" />
				<img id="day-28" src="images/30_X.png" style="left: 326px; top: 419px;" />
				<img id="day-29" src="images/30_X.png" style="left: 383px; top: 419px;" />
				<img id="day-30" src="images/30_O.png" style="left: 430px; top: 419px;" />

				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							After 30 days...
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new ThreeCasesReminder() );
			},
			audio: 'audio/30_narration.mp3',
		});
		
		for( var i = 1; i <= 30; i++ ) {
			let mark = this.root.find('#day-' + i);
			mark.css('transition','transform 0.25s ease')
				.css('transform','scale( 0.0 )');
			setTimeout( function() {
				mark.css('transform', 'scale( 1.0 )');
			}, i * 100 + 1 );
		}
	}
	
}

class ThreeCasesReminder extends LabState {
	constructor() {
		super( {
			html: `
				${DARK_BACKGROUND}
				<div style="
					position: absolute;
					width: 700px;
					left: 50px;
					bottom: 100px;
					display: grid;
					grid-template-columns: 1fr 1fr 1fr;
					align-items: center;
					justify-items: center;"
				>
					<img
						id="sack-a"
						src="images/13_SackA.png" style="
						left: 86px;
						bottom: 65px;" />
					<img
						id="sack-b"
						src="images/13_SackB.png" style="
						left: 86px;
						bottom: 65px;" />
					<img
						id="sack-c"
						src="images/13_SackC.png" style="
						left: 86px;
						bottom: 65px;" />
				</div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Recall the three treatments
							of stored corn:
						</p>
						<ul>
							<li>
								Treatment <strong>A</strong> was hung in
								the pantry.
							</li>
							<li>
								Treatment <strong>B</strong> sat on the pantry
								shelf against the wall.
							</li>
							<li>
								Treatment <strong>C</strong> was stored in the
								damp basement.
							</li>
						</ul>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new FirstSampleA() );
			},
			audio: 'audio/31_narration.mp3',
			 });
	}
	arrive() {
		super.arrive();
		this.root.find('#sack-a, #sack-b, #sack-c')
			.css('opacity', 0.25)
			.css('transition', 'opacity 1s ease');
		setTimeout( function() {
			this.root.find( '#sack-a' ).css('opacity', 1 );
		}.bind(this), 2500 );
		setTimeout( function() {
			this.root.find( '#sack-b' ).css('opacity', 1 );
		}.bind(this), 5000 );
		setTimeout( function() {
			this.root.find( '#sack-c' ).css('opacity', 1 );
		}.bind(this), 8000 );
	}
	
}

//	CHAPTER FOUR
class FirstSampleA extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/32_Backdrop.png" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Now we'll test the water activity of the corn
							that was hanging in the dry pantry, treatment
							<strong>A</strong>. Again, for precision, we 
							will need to take three samples from treatment
							<strong>A</strong>.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new PutCornAIntoGrinder() );
			},
			audio: 'audio/32_narration.mp3',
			chapter: 4,
		});
	}
	
}

class PutCornAIntoGrinder extends PutCornIntoGrinderActivity {
	constructor() {
	
		super( {
			cornImage: 'images/33_Corn.png',
			onContinue: function() {
				LAB.switchTo( new GrindCornA() );
			},
			text: `
				<p>
					As before, prepare the corn for testing. First, place a
					scoop of kernels into a small grinder.
				</p>
			`,
			audio: 'audio/33_narration.mp3',
		} );
	}
	
}

class GrindCornA extends GrindTheCornActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new TransferCornAFromGrinder() );
			},
			text: `<p>Grind up the corn.</p>`,
			audio: 'audio/34_narration.mp3',
		} );
	}
}

class TransferCornAFromGrinder extends TransferCornFromGrinderActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new SettleAndPatA() );
			},
			text: `
				<p>
					Now, add the ground corn mixture to a sample dish,
					enough to cover the bottom of the dish completely
					but not fill the dish more than halfway.
				</p>
			`,
			audio: 'audio/35_narration.mp3',
		} );
	}
}

class SettleAndPatA extends SettleAndPatActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new PutDishAInMeter() );
			},
			onInfo: function() {
				LAB.push( new UseANewSpoon() );
			},
			text: `
				<p>
					Shake or tap the dish gently to get an even distribution,
					then press down the corn with the backside of a spoon.
				</p>
			`,
			audio: 'audio/36_narration.mp3',
		} );
	}
}

class UseANewSpoon extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					Use a new spoon for each sample, to prevent contamination.
				</p>
			`,
			audio: 'audio/36_InfoBox.mp3',
		} );
	}

}

class PutDishAInMeter extends PutDishInMeterActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new WhenItBeepsItIsDoneA() );
			},
			text: `
				<p>
					Open the drawer in the water activity meter, and carefully
					place the sample dish inside. Close the door and start
					the measurement.
				</p>
			`,
			audio: 'audio/37_narration.mp3',
		} );
	}
}

class WhenItBeepsItIsDoneA extends WhenItBeepsItIsDoneActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookA() );
			},
			text: `
				<p>
					When the meter beeps, the measurement is complete.
				</p>
			`,
			audio: 'audio/38_narration.mp3',
		} );
	}
}

class RecordItInYourNotebookA extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/39_Notes.png',
			left: 175,
			top: 125,
			depth: 100,
			onContinue: function() {
				LAB.switchTo( new DiscardSampleA() );
			},
			text: `
				<p>
					Record the water activity measurement and the
					temperature in your lab notebook.
				</p>
			`,
			audio: 'audio/39_narration.mp3',
		});
	}
}

class DiscardSampleA extends DiscardSampleActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new TakeTwoMoreASamples() );
			},
			text: `
				<p>
					Then, dispose of the used sample and close the drawer.
				</p>
			`,
			audio: 'audio/40_narration.mp3',
		} );
	}
}

class TakeTwoMoreASamples extends TakeTwoSamplesActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookNumberTwoA() );
			},
			text: `
				<p>
					Test two more samples from treatment
					<strong>A</strong>.
				</p>
			`,
			audio: 'audio/41_narration.mp3',
		} );
		this.root.find('.corn-bag').attr('src','images/41_Corn_A.png')
	}
}

class RecordItInYourNotebookNumberTwoA extends ShowNotebook {
	constructor() {
		super( {
			text: `<p>
					Record the water activity measurement and the temperature
					in your lab notebook.
				</p>`,
			left: 175,
			top: 100,
			audio: 'audio/42_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new AverageSamplesFromTreatmentA() );
			},
			image: 'images/42_Notes.png',
		} );
	}
}

class AverageSamplesFromTreatmentA extends ShowNotebook {
	constructor() {
		super( {
			text: `<p>
					Average the three samples from treatment <strong>A</strong>.
				</p>`,
			left: 175,
			top: 100,
			depth: 100,
			audio: 'audio/43_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new StartSampleB() );
			},
			image: 'images/43_Notes.png',
		} );
	}
}

//	CHAPTER FIVE
class StartSampleB extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/19_Backdrop.png" />
				<img src="images/44_Corn_B.png" style="
					left: 150px;
					top: 200px;
				" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Now, we'll test the water activity of the corn that
							sat on the pantry shelf against the wall, treatment
							<strong>B</strong>. Again, for precision, we will
							need to take three samples from treatment
							<strong>B</strong>.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new PutCornBIntoGrinder() );
			},
			audio: 'audio/44_narration.mp3',
			chapter: 5,
		});
	}
	
}

class PutCornBIntoGrinder extends PutCornIntoGrinderActivity {
	constructor() {
	
		super( {
			cornImage: 'images/45_Corn.png',
			onContinue: function() {
				LAB.switchTo( new GrindCornB() );
			},
			text: `
				<p>
					Try this on your own.
				</p>
			`,
			audio: 'audio/45_narration.mp3',
		} );
		//	That balloon is pretty wide.  Narrow it.
		this.root.find('.balloon').css('width','300px');
	}
	
}

class GrindCornB extends GrindTheCornActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new TransferCornBFromGrinder() );
			},
			text: `<p>Try this on your own.</p>`,
		} );
		this.root.find('.balloon').remove();
	}
}

class TransferCornBFromGrinder extends TransferCornFromGrinderActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new SettleAndPatB() );
			},
			text: `<p>Try this on your own.</p>`,
		} );
		this.root.find('.balloon').remove();
	}
}

class SettleAndPatB extends SettleAndPatActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new PutDishBInMeter() );
			},
			text: `<p>Try this on your own.</p>`,
		} );
		this.root.find('.balloon').remove();
	}
}

class PutDishBInMeter extends PutDishInMeterActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookB() );
			},
			text: `<p>Try this on your own.</p>`,
		} );
		this.root.find('.balloon').remove();
	}
}

class RecordItInYourNotebookB extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/50_Notes.png',
			left: 175,
			top: 125,
			depth: 100,
			onContinue: function() {
				LAB.switchTo( new TakeTwoMoreSamplesB() );
			},
			text: `
				<p>
					When the meter beeps, the measurement is complete.
					Record the water activity measurement and the temperature
					in your lab notebook.
				</p>
			`,
			audio: 'audio/50_narration.mp3',
		});
	}
}

class TakeTwoMoreSamplesB extends TakeTwoSamplesActivity {
	constructor() {
		super( {
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookNumberTwoB() );
			},
			text: `
				<p>
					Remove and dispose of the sample and test the other two
					samples from treatment <strong>B</strong>.
				</p>
			`,
			audio: 'audio/51_narration.mp3',
		} );
		this.root.find('.corn-bag').attr('src','images/51_Corn.png')
	}
}

class RecordItInYourNotebookNumberTwoB extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/52_Notes.png',
			left: 175,
			top: 125,
			onContinue: function() {
				LAB.switchTo( new AverageSamplesFromTreatmentB() );
			},
			text: `
				<p>
					Record the water activity measurement and the temperature
					in your lab notebook.
				</p>
			`,
			audio: 'audio/52_narration.mp3',
		});
	}
}

class AverageSamplesFromTreatmentB extends ShowNotebook {
	constructor() {
		super( {
			text: `<p>
					Average the three samples from treatment <strong>B</strong>.
				</p>`,
			left: 175,
			top: 100,
			depth: 100,
			audio: 'audio/53_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new TakeThreeSamplesActivity() );
			},
			image: 'images/53_Notes.png',
		} );
	}
}

//	CHAPTER SIX
class TakeThreeSamplesActivity extends LabState {
	constructor( config ) {
		super( {
			html: `
				<style>
					.sample-scene {
						position: absolute;
						left: 0px;
						transition: left 10s ease;
					}
					.sample-scene > * {
						position: absolute;
						transition: all 0.5s ease;
					}
					#scene-two {
						top: 300px;
					}
					#scene-three {
						top: 600px;
					}
					#all-scenes {
						position: absolute;
						transition: top 6s ease;
						top: 0px;
					}
				</style>
				<div id="all-scenes">
					<div id="scene-one" class="sample-scene"></div>
					<div id="scene-two" class="sample-scene"></div>
					<div id="scene-three" class="sample-scene"></div>
				</div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							We'll test the water activity of the corn that was
							stored in the damp basement, treatment
							<strong>C</strong>.  Again, for precision, we will
							need to take three samples from treatment
							<strong>C</strong>.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new RecordItInYourNotebookC() );
			},
			audio: 'audio/54_narration.mp3',
			chapter: 6,
		});
		this.animations = [
			new GrindAnimation( this.root.find('#scene-one') ),
			new GrindAnimation( this.root.find('#scene-two') ),
			new GrindAnimation( this.root.find('#scene-three') ),
		];
		this.root.find('.corn-bag').attr('src','images/54_Corn.png')
	}
	
	arrive() {
		super.arrive();
		this.animations[0].animate();
		setTimeout( function() { this.animations[1].animate(); }.bind(this), 6000 );
		setTimeout( function() { this.animations[2].animate(); }.bind(this), 12000 );
		setTimeout( function() {
			this.animations[1].animate();
			this.root.find('#all-scenes').css('top', '-300px');
		}.bind(this), 6000 );
	}
	
}

class RecordItInYourNotebookC extends ShowNotebook {
	constructor() {
		super( {
			image: 'images/55_Notes.png',
			left: 175,
			top: 125,
			onContinue: function() {
				LAB.switchTo( new AverageSamplesFromTreatmentC() );
			},
			text: `
				<p>
					Record the water activity measurement and the temperature
					in your lab notebook.
				</p>
			`,
			audio: 'audio/55_narration.mp3',
		});
	}
}

class AverageSamplesFromTreatmentC extends ShowNotebook {
	constructor() {
		super( {
			text: `<p>
					Average the three samples from treatment <strong>C</strong>.
				</p>`,
			left: 175,
			top: 100,
			depth: 100,
			audio: 'audio/56_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new ThreeResults() );
			},
			image: 'images/56_Notes.png',
		} );
	}
}

class ThreeResults extends LabState {
	constructor() {
		super( {
			html: `
				${DARK_BACKGROUND}
				<style>
					#result-grid {
						position: absolute;
						width: 700px;
						left: 50px;
						top: 200px;
						display: grid;
						grid-template-columns: 1fr 1fr 1fr;
						grid-column-gap: 1em;
						grid-row-gap: 5px;
						align-items: center;
						justify-items: center;
						color: white;
					}
					#result-grid p {
						text-align: center;
						font-size: 17px;
						margin: 0px;
					}
					#result-grid .result {
						font-size: 32px;
					}
				</style>
				<div id="result-grid">
					<img
						id="sack-a"
						src="images/13_SackA.png" />
					<img
						id="sack-b"
						src="images/13_SackB.png" />
					<img
						id="sack-c"
						src="images/13_SackC.png" />
					<p>
						Hanging in a dry pantry
					</p>
					<p>
						Shelf in a dry pantry
					</p>
					<p>
						Shelf in a damp basement
					</p>
					<p class="result">
						a<sub>w</sub> 0.25
					</p>
					<p class="result">
						a<sub>w</sub> 0.45
					</p>
					<p class="result">
						a<sub>w</sub> 0.60
					</p>
					<p>
						<button id="treatment-a">Treatment <b>A</b></button>
					</p>
					<p>
						<button id="treatment-b">Treatment <b>B</b></button>
					</p>
					<p>
						<button id="treatment-c">Treatment <b>C</b></button>
					</p>
				</div>
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Take a look at the average water activity of the
							samples. Which method of storage do you think will
							maintain safety and quality of the dried corn the
							longest?
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			audio: 'audio/57_narration.mp3',
			 });
	}
	
	arrive() {
		super.arrive();
		this.root.find( '#treatment-a' ).click( function() {
			this.didClickTreatmentA();
		}.bind(this));
		this.root.find( '#treatment-b' ).click( function() {
			this.didClickTreatmentB();
		}.bind(this));
		this.root.find( '#treatment-c' ).click( function() {
			this.didClickTreatmentC();
		}.bind(this));
	}
	
	keyDown( e ) {
		switch( e.key ) {
			case 'a':
				this.didClickTreatmentA();
				return true;
			case 'b':
				this.didClickTreatmentB();
				return true;
			case 'c':
				this.didClickTreatmentC();
				return true;
		}
		return super.keyDown( e );
	}
	
	didClickTreatmentA() {
		LAB.push( new TreatmentAGood(), function() {
			LAB.jumpTo( new SampleASummary() );
		} );
	}
	
	didClickTreatmentB() {
		LAB.push( new TreatmentBBad() );
	}
	
	didClickTreatmentC() {
		LAB.push( new TreatmentCBad() );
	}
	
}

class TreatmentAGood extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<h3 style="padding-bottom: 0px;">Treatment A</h3>
				<p>
					Yes, that is correct. Treatment <strong>A</strong>'s
					low average water activity indicates the dried corn
					hanging in a bag in a dry pantry did not absorb water
					from the air.
					This maintains the water activity of 0.2
					to 0.3 required by most safety and quality standards.
				</p>
				<p>
					However, this is only for 30 days. More research is
					needed for longer term storage.
				</p>
			`,
			audio: 'audio/57_Response1.mp3',
		} );
	}

}

class TreatmentBBad extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<h3 style="padding-bottom: 0px;">Treatment B</h3>
				<p>
					This is not a recommended method for storage, since the
					water activity increased in this storage environment to
					a level higher than recommended for safety and quality
					of dried vegetables.
				</p>
			`,
			audio: 'audio/57_Response2.mp3',
		} );
	}

}

class TreatmentCBad extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<h3 style="padding-bottom: 0px;">Treatment C</h3>
				<p>
					This is not a recommended method for storage, since the
					water activity increased in this storage environment to
					a level higher than recommended for safety and quality
					of dried vegetables.
				</p>
			`,
			audio: 'audio/57_Response3.mp3',
		} );
	}

}

class SampleASummary extends LabState {
	constructor() {
		super( {
			html: `
				<img src="images/32_Backdrop.png" />
				<div class="balloon without-portrait"
					style="
						right: 80px;
						top: 20px;
						width: 550px;
					"
				>
					<div class="dialogue">
						<p>
							Corn under treatments <strong>B</strong> and
							<strong>C</strong> did not maintain the water
							activity of 0.20 to 0.30 required by most
							safety and quality standards.
						</p>
						<p>
							This demonstrates
							that when storing dried foods, storage conditions
							must be controlled in order to maintain safe
							water activity levels.
						</p>
					</div>
				</div>
				${HOME_BUTTON}
			`,
			onContinue: function() {
				LAB.switchTo( new AroundTheWorld() );
			},
			audio: 'audio/58_narration.mp3',
			 });
	}
	
}

//	CHAPTER SEVEN
class AroundTheWorld extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Around the world, traditional and conventional methods of
					drying and preserving food are based on the same
					principle: controlling water activity.
				</p>
				`,
			audio: 'audio/59_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new HistoryOfStoringFood() );
			},
			onInfo: function() {
				LAB.push( new OtherVirtualLabs() );
			},
			chapter: 7,
		} );
	}
}

class OtherVirtualLabs extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					This Virtual Lab focuses on controlling water activity in
					foods by removing the water (which can be done using
					dehydration, evaporation, freeze-drying, or concentrating).
					The Virtual Lab <em>Understanding Water Activity</em>
					addressed additional ways of controlling water activity,
					such as binding up water by adding salt or sugar.
				</p>
			`,
			audio: 'audio/59_InfoBox.mp3',
		} );
	}

}

class HistoryOfStoringFood extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Ever since humans began storing food, they have manipulated
					its water activity. People didn't always use the term
					"water activity," but they understood that removing or
					binding water makes food last longer without spoiling and
					also makes it easier to grind.
				</p>
				`,
			audio: 'audio/60_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new LeaveLessToChance() );
			},
			onInfo: function() {
				LAB.push( new StorageMethodsImportant() );
			},
		} );
	}
}

class StorageMethodsImportant extends InfoBoxState {

	constructor( config ) {
		super( {
			html: `
				<p>
					How you store food is also important. Native Americans and
					other peoples around the world have used various methods
					over time, including jars made of pottery or birch bark, to
					protect dried corn – and seed corn – from moisture and
					contaminants. Corn kernels stored in a jar at Taos Pueblo
					in New Mexico were still able to sprout after 200 years!
				</p>
				<p>
					Today, we often seal foods in plastic or store them in
					glass jars.
				</p>
			`,
			audio: 'audio/60_InfoBox.mp3',
		} );
	}

}

class LeaveLessToChance extends LabTechSpeaking {
	constructor() {
		super( {
			html: `
				<p>
					Now that scientists can measure water activity with lab
					equipment, we leave less to chance. We can test food to
					make sure it's safe and stable and design effective storage
					and packaging.
				</p>
				`,
			audio: 'audio/61_narration.mp3',
			onContinue: function() {
				LAB.switchTo( new EndLabState() );
			},
		} );
	}
}

class EndLabState extends EndScreen {
	constructor() {
		super( {
			placard: 'images/ControllingWaterActivityPlacard.png',
			title: 'Controlling Water Activity',
			firstState: new WelcomeState(),
			audio: 'audio/end.mp3',
		} );
	}
}

